/*
 * Copyright (c) 2022 Luca Fulchir<luker@fenrirproject.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

use crate::errors::MkError;
use crate::Output;

/// options to pass to `mkswap`
#[derive(Clone, PartialEq, Eq, PartialOrd, Ord)]
pub enum Options {
    /// `--check` check bad blocks before creating the swap area
    Check,
    /// `--force` allow swap size area be larger than device
    Force,
    /// `--pagesize` specify page size in bytes
    Pagesize(u64),
    /// `--label LABEL` specify label
    Label(String),
    /// `--swapversion NUM` specify swap-space version number
    SwapVersion(u32),
    /// `--uuid UUID` specify the uuid to use
    Uuid(::uuid::Uuid),
    /// `--lock=yes` use exclusive device lock
    ExclusiveLock,
    /// `--lock=no or nonblock` use NONexclusive device lock
    NonExclusiveLock,
}

impl ToString for Options {
    fn to_string(&self) -> String {
        match self {
            Options::Check => "--check".to_owned(),
            Options::Force => "--force".to_owned(),
            Options::Pagesize(p) => "--pagesize ".to_owned() + &p.to_string(),
            Options::Label(l) => "--label \"".to_owned() + &l + "\"",
            Options::SwapVersion(v) => {
                "--swapversion ".to_owned() + &v.to_string()
            }
            Options::Uuid(u) => "--uuid ".to_owned() + &u.to_string(),
            Options::ExclusiveLock => "--lock=yes".to_owned(),
            Options::NonExclusiveLock => "--lock=no".to_owned(),
        }
    }
}

/// Wrapper on `mkswap`
pub(in crate) struct Swap {}

impl Swap {
    /// write the filesystem to disk
    #[must_use]
    pub fn write(
        options: &Vec<Options>,
        dev: &::udev::Device,
        output: Output,
    ) -> Result<(), MkError> {
        let mut opts: Vec<Options> = options.to_vec();
        opts.sort();
        opts.dedup();
        let mut str_opts = Vec::<String>::with_capacity(opts.len() + 1);
        for opt in opts.into_iter() {
            str_opts.push(opt.to_string());
        }
        let node = super::dev_to_node(dev)?;
        str_opts.push(node);
        let (out, err) = output.to_pair();
        super::run("mkswap", str_opts, out, err)
    }
}
